/*
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <njpatel@gmail.com>
 */

public class TabBar : Gtk.Alignment
{
  private Gtk.VBox _vbox;
  private Gtk.HBox _tab_box;
  private Gtk.HBox _toolbar_box;

  private TabBarItem? _active_item = null;

  public signal void tab_changed (TabBarItem item);

  public TabBar ()
  {
    Object ();
  }

  construct
  {
    get_style_context ().add_class ("menubar");
    set_padding (0, 4, 0, 0);

    _vbox = new Gtk.VBox (false, 8);
    add (_vbox);

    _tab_box = new Gtk.HBox (false, 0);
    _vbox.pack_start (_tab_box, false, false, 0);

    _toolbar_box = new Gtk.HBox (false, 0);
    _vbox.pack_start (_toolbar_box, false, true, 0);

    show_all ();
  }

  public override bool draw (Cairo.Context cr)
  {
    cr.set_operator (Cairo.Operator.CLEAR);
    cr.paint ();

    cr.set_operator (Cairo.Operator.OVER);

    // Menu background
    Gtk.render_background (get_style_context (),
                           cr,
                           0,
                           0,
                           get_allocated_width (),
                           get_allocated_height ());

    // Arrow
    if (_active_item is Gtk.Widget)
    {
      Gtk.Allocation a;
      _active_item.get_allocation (out a);

      cr.set_line_width (1.0);
      cr.set_source_rgb (1.0f, 1.0f, 1.0f);

      // Main Arrow
      cr.move_to (a.x + a.width/2.0, get_allocated_height () - 5);
      cr.line_to (a.x + a.width/2.0 + 5, get_allocated_height ());
      cr.line_to (a.x + a.width/2.0 - 5, get_allocated_height ());
      cr.close_path ();
      
      cr.fill ();

      if (_active_item.get_toolbar () is Gtk.Widget)
      {
        cr.save ();
        cr.translate (0.5, 0.5);

        draw_arrowed_line (cr, 0, a.height, get_allocated_width (), (int)(a.x + a.width/2.0));
        cr.set_source_rgba (1.0f, 1.0f, 1.0f, 0.2f);
        cr.stroke ();

        draw_arrowed_line (cr, 0, a.height-1, get_allocated_width (), (int)(a.x + a.width/2.0));
        cr.set_source_rgba (0.0f, 0.0f, 0.0f, 0.5f);
        cr.stroke ();

        cr.restore ();
      }
    }

    propagate_draw (_vbox, cr);

    return true;
  }

  private void draw_arrowed_line (Cairo.Context cr, int x, int y, int width, int point)
  {
    int size = 5;

    cr.move_to (x, y);
    cr.line_to (point - size, y);
    cr.line_to (point, y - size);
    cr.line_to (point + size, y); 
    cr.line_to (x + width, y);
  }

  public void add_item (TabBarItem item)
  {
    item.clicked.connect (on_tab_clicked);
    _tab_box.pack_start (item, false, false, 0);

    if (item.get_toolbar () is Gtk.Widget)
    {
      var toolbar = item.get_toolbar ();
      toolbar.set_no_show_all (true);
      toolbar.hide ();
      _toolbar_box.pack_start (toolbar);
    }

    queue_draw ();
  }

  public void on_tab_clicked (Gtk.Button? gtk_item)
  {
    TabBarItem? item = gtk_item as TabBarItem;

    if (_active_item == item || gtk_item == null)
      return;
    
    if (_active_item is TabBarItem)
    {
      _active_item.active = false;
      _active_item.get_toolbar ().hide ();
    }

    _active_item = item;
    _active_item.active = true;
    if (_active_item.get_toolbar () is Gtk.Widget)
      _active_item.get_toolbar ().show ();

    tab_changed (item);

    queue_draw ();
  }
}
